local RUF = RUF or LibStub("AceAddon-3.0"):GetAddon("RUF")
local L = LibStub("AceLocale-3.0"):GetLocale("RUF")
local LSM = LibStub("LibSharedMedia-3.0")
local _, ns = ...
local oUF = ns.oUF

--[[
    ## Options
.disableMouse       - Disables mouse events (boolean)
.disableCooldown    - Disables the cooldown spiral (boolean)
.size               - Aura icon size. Defaults to 16 (number)
.onlyShowPlayer     - Shows only auras created by player/vehicle (boolean)
.showStealableBuffs - Displays the stealable texture on buffs that can be stolen (boolean)
.spacing            - Spacing between each icon. Defaults to 0 (number)
.['spacing-x']      - Horizontal spacing between each icon. Takes priority over `spacing` (number)
.['spacing-y']      - Vertical spacing between each icon. Takes priority over `spacing` (number)
.['growth-x']       - Horizontal growth direction. Defaults to 'RIGHT' (string)
.['growth-y']       - Vertical growth direction. Defaults to 'UP' (string)
.initialAnchor      - Anchor point for the icons. Defaults to 'BOTTOMLEFT' (string)
.filter             - Custom filter list for auras to display. Defaults to 'HELPFUL' for buffs and 'HARMFUL' for
                      debuffs (string)

    ## Options Auras
.numBuffs     - The maximum number of buffs to display. Defaults to 32 (number)
.numDebuffs   - The maximum number of debuffs to display. Defaults to 40 (number)
.numTotal     - The maximum number of auras to display. Prioritizes buffs over debuffs. Defaults to the sum of
                .numBuffs and .numDebuffs (number)
.gap          - Controls the creation of an invisible icon between buffs and debuffs. Defaults to false (boolean)
.buffFilter   - Custom filter list for buffs to display. Takes priority over `filter` (string)
.debuffFilter - Custom filter list for debuffs to display. Takes priority over `filter` (string)

    ## Options Buffs
.num - Number of buffs to display. Defaults to 32 (number)

    ## Options Debuffs
.num - Number of debuffs to display. Defaults to 40 (number)

## Attributes
button.caster   - the unit who cast the aura (string)
button.filter   - the filter list used to determine the visibility of the aura (string)
button.isDebuff - indicates if the button holds a debuff (boolean)
button.isPlayer - indicates if the aura caster is the player or their vehicle (boolean)
]]--

function RUF.SetBuffs(self, unit)

        -- Position and size
        local Buffs = CreateFrame('Frame', nil, self)
        Buffs:SetPoint('BOTTOMLEFT', self, 'TOPLEFT',0,2)
        Buffs:SetSize(32 * 5, 32 * 2 + 2) -- x,y size of buff holder frame

        Buffs.size = 32
        Buffs["growth-x"] = "RIGHT"
        Buffs["growth-y"] = "UP"
        Buffs.initialAnchor = "BOTTOMLEFT" -- Where icons spawn from in the buff frame
        Buffs["spacing-x"] = -1
        Buffs["spacing-y"] = 2
    
        -- Register with oUF
        self.Buffs = Buffs
end

function RUF.SetDebuffs(self, unit)

    -- Position and size
    local Debuffs = CreateFrame('Frame', nil, self)
    Debuffs:SetPoint('BOTTOMRIGHT', self, 'TOPRIGHT',0,2)
    Debuffs:SetSize(32 * 5, 32 * 2 + 2) -- x,y size of buff holder frame

    Debuffs.size = 32
    Debuffs["growth-x"] = "LEFT"
    Debuffs["growth-y"] = "UP"
    Debuffs.initialAnchor = "BOTTOMRIGHT" -- Where icons spawn from in the buff frame
    Debuffs["spacing-x"] = -1
    Debuffs["spacing-y"] = 2

    -- Register with oUF
    self.Debuffs = Debuffs
end

-- TODO:
-- AURA Bars, use oUF_Lumen.
--[[
      if self.cfg.auras.barTimers.show then
    local barTimers = auras:CreateBarTimer(self, 12, 12, 24, 2)
    barTimers:SetPoint("BOTTOMLEFT", self, "TOPLEFT", -2, cfg.frames.secondary.height + 16)
    barTimers.initialAnchor = "BOTTOMLEFT"
    barTimers["growth-y"] = "UP"
    barTimers.CustomFilter = PlayerCustomFilter
    barTimers.PostUpdateIcon = PostUpdateBarTimer
    self.Buffs = barTimers
  end
]]--

--[[-- Post Update BarTimer Aura
local PostUpdateBarTimer = function(element, unit, button, index)
  local name, _, count, dtype, duration, expirationTime = UnitAura(unit, index, button.filter)

  if duration and duration > 0 then
    button.timeLeft = expirationTime - GetTime()
    button.bar:SetMinMaxValues(0, duration)
    button.bar:SetValue(button.timeLeft)

    if button.isDebuff then -- bar color
      button.bar:SetStatusBarColor(1, 0.1, 0.2)
    else
      button.bar:SetStatusBarColor(0, 0.4, 1)
    end
  else
    button.timeLeft = math.huge
    button.bar:SetStatusBarColor(0.6, 0, 0.8) -- permenant buff / debuff
  end

  button.spell:SetText(name) -- set spell name

  button:SetScript('OnUpdate', function(self, elapsed)
    auras:BarTimer_OnUpdate(self, elapsed)
  end)
end]]--

--[[local _, ns = ...

local auras = ns.auras
local core, cfg, m, oUF = ns.core, ns.cfg, ns.m, ns.oUF

local max = max

-- ------------------------------------------------------------------------
-- > BARTIME AURAS RELATED FUNCTIONS
-- ------------------------------------------------------------------------

function auras:BarTimer_OnUpdate(icon, elapsed)
	if icon.timeLeft then
		icon.timeLeft = max(icon.timeLeft - elapsed, 0)
		icon.bar:SetValue(icon.timeLeft) -- update the statusbar

		-- text color
		if icon.timeLeft > 0 and icon.timeLeft < 60 then
			icon.time:SetFormattedText(core:formatTime(icon.timeLeft))
			if icon.timeLeft < 6 then
				icon.time:SetTextColor(1, 0.25, 0.25)
			elseif icon.timeLeft < 10 then
					icon.time:SetTextColor(1, 0.9, 0.5)
			else
				icon.time:SetTextColor(1, 1, 1)
			end
		elseif icon.timeLeft > 60 and icon.timeLeft < 60 * 5 then
			icon.time:SetTextColor(1, 1, 1)
			icon.time:SetFormattedText(core:formatTime(icon.timeLeft))
		else
			icon.time:SetText()
		end
	end
end

local SortAuras = function(a, b)
    if a and b and a.timeLeft and b.timeLeft then
    	return a.timeLeft > b.timeLeft
    end
end

local PreSetPosition = function(Auras)
	table.sort(Auras, SortAuras)
	return 1
end

local PostCreateBar = function(Auras, button)
  button.icon:SetTexCoord(0, 1, 0, 1)

  button.overlay:SetTexture(m.textures.border)
  button.overlay:SetTexCoord(0, 1, 0, 1)
  button.overlay.Hide = function(self) self:SetVertexColor(0.3, 0.3, 0.3) end

	button.bar = CreateFrame('StatusBar', nil, button)
	button.bar:SetStatusBarTexture(m.textures.status_texture)
	button.bar:SetPoint("TOPLEFT", button, 'TOPRIGHT', 4, -2)
	button.bar:SetHeight(Auras.size - 4)
	button.bar:SetWidth(cfg.frames.main.width - Auras.size - 2)
	core:setBackdrop(button.bar, 2, 2, 2, 2)

	button.bar.bg = button.bar:CreateTexture(nil, 'BORDER')
	button.bar.bg:SetAllPoints()
	button.bar.bg:SetAlpha(0.3)
	button.bar.bg:SetTexture(m.textures.bg_texture)
	button.bar.bg:SetColorTexture(1/3, 1/3, 1/3)

	button.spell = button.bar:CreateFontString(nil, 'OVERLAY')
	button.spell:SetPoint("LEFT", button.bar, "LEFT", 4, 0)
	button.spell:SetFont(m.fonts.font_big, 16, "THINOUTLINE")
	button.spell:SetWidth(button.bar:GetWidth() - 25)
	button.spell:SetTextColor(1, 1, 1)
	button.spell:SetShadowOffset(1, -1)
	button.spell:SetShadowColor(0, 0, 0, 1)
	button.spell:SetJustifyH("LEFT")
	button.spell:SetWordWrap(false)

	button.time = button.bar:CreateFontString(nil, 'OVERLAY')
	button.time:SetPoint("RIGHT", button.bar, "RIGHT", -4, 0)
	button.time:SetFont(m.fonts.font, 12, "THINOUTLINE")
	button.time:SetTextColor(1, 1, 1)
	button.time:SetShadowOffset(1, -1)
	button.time:SetShadowColor(0, 0, 0, 1)
	button.time:SetJustifyH('RIGHT')

	button.count:ClearAllPoints()
	button.count:SetFont(m.fonts.font, 12, 'OUTLINE')
	button.count:SetPoint('TOPRIGHT', button, 3, 3)
end

function auras:CreateBarTimer(self, num, rows, size, spacing)
	local auras = CreateFrame("Frame", nil, self)
	auras:SetSize( (num * (size + 9)) / rows, (size + 9) * rows)
	auras.num = num
	auras.size = size
	auras.spacing = spacing or 4
	auras.disableCooldown = true
	auras.PreSetPosition = PreSetPosition  -- sort auras by time remaining
	auras.PostCreateIcon = PostCreateBar -- set overlay, cd, count, timer
	return auras
end
]]--

